package driver;

import ir.MyVisitor;
import org.antlr.v4.runtime.CharStreams;
import org.antlr.v4.runtime.CommonTokenStream;
import org.antlr.v4.runtime.tree.ParseTree;
import parser.MySysYVisitor;
import parser.SysYLexer;
import parser.SysYParser;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

public class CompilerRun {
    //处理命令行参数运行
    public static void run(String [] args) {
        List<String> list = Arrays.asList(args);
        Iterator<String > it = list.iterator();
        String sourceFile = null;
        String targetFile = null;
        while (it.hasNext()) {
            if (it.next().equals("-S")) {
               sourceFile = it.next();
            }
            if (it.next().equals("-o")) {
               targetFile = it.next();
            }
        }
        if (sourceFile == null) {
            System.out.println("No input file.");
            return;
        }
        if (targetFile == null) {
            System.out.println("No output file.");
            return;
        }
        compiler(sourceFile, targetFile);
    }

    public static void compiler(String sourceFile, String tagetFile) {
        try {
            var input = CharStreams.fromFileName(sourceFile);
            PrintStream printStream = new PrintStream(tagetFile);
            SysYLexer lexer = new SysYLexer(input);
            CommonTokenStream tokens = new CommonTokenStream(lexer);
            SysYParser parser = new SysYParser(tokens);
            ParseTree tree = parser.compUnit();
            var visitor = new MySysYVisitor();
            visitor.visit(tree);
            var r = visitor.getRoot();
            System.out.println(r);
            MyVisitor visitor1 = new MyVisitor(r);
            visitor1.compiler();
            System.setOut(printStream);
            //visitor1.setName();
            System.out.println(visitor1.getModule());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
