package ir;

import ir.instruction.AllocaInstr;
import ir.instruction.BinaryInstr;
import ir.instruction.BrInstr;
import ir.instruction.CallInstr;
import ir.instruction.CastInstr;
import ir.instruction.GetElemPtrInstr;
import ir.instruction.Instr;
import ir.instruction.LoadInstr;
import ir.instruction.PhiInstr;
import ir.instruction.RetInstr;
import ir.instruction.StoreInstr;
import ir.type.ArraySysYType;
import ir.type.ConstSysYType;
import ir.type.IntSysYType;
import ir.type.PointerSysYType;
import ir.type.SysYType;
import ir.value.BasicBlock;
import ir.value.Function;
import ir.value.GlobalValue;
import ir.value.InitVal;
import ir.value.Value;

//build工厂，可以产生instr，basicBlock, function等等

import java.util.ArrayList;
import java.util.BitSet;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

public class MyIrBuilder {
    void setInsertPoint(Value value) {

    }
    public BasicBlock getBasicBlock() {
        return new BasicBlock(new ArrayList<>(), new ArrayList<>());
    }

    public CallInstr getCallInstr(Function function, ArrayList<Value> pramas) {
        return new CallInstr(function, pramas);
    }

    public void buildCallInstr(Function function, ArrayList<Value> pramas, BasicBlock bb) {
        CallInstr callInstr = getCallInstr(function, pramas);
        pramas.forEach(prama->{
            prama.addUser(callInstr);
                }
        );
        bb.addLast(getCallInstr(function, pramas));

    }

    public Function getFunction(SysYType type) {
        return new Function(type);
    }

    public Function getFunction(ArrayList<BasicBlock> basicBlocks, SysYType type) {
        return new Function(basicBlocks, type);
    }

    public GetElemPtrInstr getGep(Value base, ArrayList<Value> indexs, SysYType ptr) {
        return new GetElemPtrInstr(base, indexs, ptr);
    }

    public InitVal getZero() {
        InitVal initVal = new InitVal(new ArrayList<>(), IntSysYType.construct());
        initVal.setName("0");
        return initVal;
    }

    public void buildGep(Value base, ArrayList<Value> indexs, BasicBlock bb) {
        PointerSysYType ptr = (PointerSysYType) base.getType();
        SysYType type;
        if (ptr.getParam() instanceof ConstSysYType) {
            type = ((ConstSysYType) ptr.getParam()).getParam();
            type = ((ArraySysYType) type).getParam();
            ptr = new PointerSysYType(type);
        } else if(ptr.getParam() instanceof ArraySysYType) {
            type = ((ArraySysYType) ptr.getParam()).getParam();
            ptr = new PointerSysYType(type);
        } else {
            buildLoadInstr(base, bb);
            base = bb.getList().getLast().getValue();
            type = ((PointerSysYType) ptr.getParam()).getParam();
            ptr = new PointerSysYType(type);
            indexs = new ArrayList<>(indexs.subList(1, indexs.size()));
          //  indexs.add(getZero());
            if (indexs.size() > 1) {
                buildGep(base, indexs, bb);
            } else {
                Instr instr = getGep(base, indexs, ptr);
                base.addUser(instr);
                bb.addLast(instr);
            }
            return;
        }
        boolean loadFlag = false;
        for (int i = 0; i < indexs.size() - 2; i++) {
            if (ptr.getParam() instanceof ConstSysYType) {
                type = ((ConstSysYType) ptr.getParam()).getParam();
            } else {
                type = ptr.getParam();
            }
            if (!(type instanceof ArraySysYType)) {
                indexs.remove(indexs.size() - 1);
                loadFlag = true;
                break;
            }
            type = ((ArraySysYType) type).getParam();
            ptr = new PointerSysYType(type);
        }
        Instr instr = getGep(base, indexs, ptr);
        base.addUser(instr);
        bb.addLast(instr);
        if (loadFlag) {
            buildLoadInstr(bb.getList().getLast().getValue(), bb);
        }
    }

    public IntSysYType getIntSysType() {
        return IntSysYType.construct();
    }

    public BinaryInstr getBinaryInstr(BinaryInstr.Type tag, Value l, Value r) {
        return new BinaryInstr(tag, l.getType(), l, r);
    }

    public BinaryInstr getBinaryInstr(BinaryInstr.Type tag, Value l) {
        return new BinaryInstr(tag, l.getType(), l);
    }

    public BinaryInstr getCmpInstr(BinaryInstr.Type cmp, Value l, Value r) {
        return new BinaryInstr(BinaryInstr.Type.ICmp, l.getType(), l, r, cmp);
    }

    public void buildCmpInstr(BinaryInstr.Type cmp, Value l, Value r, BasicBlock bb) {
        Instr instr = getCmpInstr(cmp, l, r);
        l.addUser(instr);
        r.addUser(instr);
        bb.addLast(instr);
    }

    public void buildBinaryInstr(BinaryInstr.Type tag, Value l, BasicBlock bb) {
        Instr instr = getBinaryInstr(tag, l);
        l.addUser(instr);
        bb.addLast(instr);
    }

    public void buildBinaryInstr(BinaryInstr.Type tag, Value l, Value r, BasicBlock bb) {
        Instr instr = getBinaryInstr(tag, l, r);
        l.addUser(instr);
        r.addUser(instr);
        bb.addLast(instr);
    }

    public InitVal getInitVal(SysYType type) {
        return new InitVal(new ArrayList<>(), type);
    }

    public RetInstr getRetInstr(SysYType type) {
        return new RetInstr(type, "");
    }



    public void buildRetInstr(Value retValue, BasicBlock bb) {
        SysYType type = retValue.getType();
        RetInstr retInstr = getRetInstr(type);
        retInstr.addUse(retValue);
        bb.addLast(retInstr);
    }

    public GlobalValue getGlobalValue(SysYType type, String name) {
        return new GlobalValue(PointerSysYType.construct(type), name);
    }


    public BrInstr getBrInstr(Value conv, BasicBlock basicBlock1, BasicBlock basicBlock2) {
        return new BrInstr(conv, basicBlock1, basicBlock2);
    }

    public BrInstr getBrInstr(BasicBlock basicBlock) {
        return new BrInstr(basicBlock);
    }

    public void buildBrInstr(BasicBlock basicBlock, BasicBlock bb) {
        Instr instr = getBrInstr(basicBlock);
        basicBlock.addUser(instr);
        bb.addLast(instr);
    }

    public void buildBrInstr(Value conv, BasicBlock basicBlock1, BasicBlock basicBlock2, BasicBlock bb) {
        Instr instr = getBrInstr(conv, basicBlock1, basicBlock2);
        basicBlock1.addUser(instr);
        basicBlock2.addUser(instr);
        bb.addLast(instr);
    }

    public LoadInstr getLoadInstr(Value ptr) {
        return new LoadInstr(ptr);
    }

    public void buildLoadInstr(Value ptr, BasicBlock bb) {
        Instr instr = getLoadInstr(ptr);
        ptr.addUser(instr);
        bb.addLast(instr);
    }

    public StoreInstr getStoreInstr(Value target, Value ptr) {
        return new StoreInstr(target, ptr);
    }

    public AllocaInstr getAllocaInstr(SysYType type) {
        return new AllocaInstr(type);
    }

    public void buildAllocaInstr(SysYType type, BasicBlock bb) {
        bb.addLast(getAllocaInstr(type));
    }


    public void buildStoreInstr(Value target, Value ptr, BasicBlock bb) {
        bb.addLast(getStoreInstr(target, ptr));
    }

    public CastInstr getCastInstr(Value source, SysYType type) {
        return new CastInstr(source, type);
    }

    public void buildCastInstr(Value source, SysYType type, BasicBlock bb) {
        Instr instr = getCastInstr(source, type);
        source.addUser(instr);
        bb.addLast(instr);
    }

    public PhiInstr getPhiInstr(HashMap<BasicBlock, Value> values) {
        Iterator<Value> it = values.values().iterator();
        return new PhiInstr(it.next().getType(), values);
    }
}
