package ir;

import ir.instruction.BinaryInstr;
import ir.instruction.Instr;
import ir.type.ArraySysYType;
import ir.type.ConstSysYType;
import ir.type.FloatSysYType;
import ir.type.FunctionSysYType;
import ir.type.IntSysYType;
import ir.type.PointerSysYType;
import ir.type.SysYType;
import ir.type.VoidSysYType;
import ir.value.BasicBlock;
import ir.value.CompileUnit;
import ir.value.Function;
import ir.value.GlobalValue;
import ir.value.InitVal;
import ir.value.NameSpace;
import ir.value.User;
import ir.value.Value;
import treeir.Assign;
import treeir.Break;
import treeir.Condition;
import treeir.Continue;
import treeir.FunCall;
import treeir.FuncDef;
import treeir.Ident;
import treeir.Index;
import treeir.Loop;
import treeir.MyArray;
import treeir.Node;
import treeir.NumberFloat;
import treeir.NumberInt;
import treeir.Operation;
import treeir.Return;
import treeir.Root;
import treeir.Sequence;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
/*
    通过遍历根节点获得llvm_ir
    最终生成Module
 */



public class MyVisitor {

    private MyIrBuilder buider = new MyIrBuilder();
    private CompileUnit compileUnit = CompileUnit.getInstance();
    private BasicBlock bb = buider.getBasicBlock();
    private MyModule module = new MyModule();
            private HashMap<String, SysYType> functions;
    private HashMap<String, SysYType> globals;
    private Root root;

    public BasicBlock getBb() {
        return bb;
    }

    public MyVisitor(BasicBlock bb) {
        this.bb = bb;
    }

    public Value visitArray(Node node, BasicBlock bb, Function function, HashMap<String, SysYType> locals,
                             Value array) {    //获得数组的初始化值
        User value;
        if (((ArraySysYType)((MyArray) node).getType()).getParam() instanceof IntSysYType) {
            value = new InitVal.SysYIntArray(new ArrayList<>(), ((MyArray) node).getType());
        } else {
            value = new InitVal.SysYFloatArray(new ArrayList<>(), ((MyArray) node).getType());
        }
        int i = 0;
        LinkedList<Integer> dim = new LinkedList<>();
        SysYType curenArray = ((MyArray) node).getType();
        while (curenArray instanceof ArraySysYType) {
            ArraySysYType arraySysYType = (ArraySysYType) curenArray;
            dim.addFirst(arraySysYType.getSize());
            curenArray = arraySysYType.getInner();
        }
        for (Node child : node.getChildren()) {
            Value element = visitVar(child, bb, function, locals);
            if (element instanceof InitVal) {
                value.addUse(element);
            } else {
                InitVal initVal = new InitVal(new ArrayList<>(), ConstSysYType.construct(IntSysYType.construct()));
                value.addUse(initVal);
                initVal.setName(String.valueOf(0));
                ArrayList<Value> instrIndex = new ArrayList<>();
                instrIndex.add(buider.getZero());
                int num = i;
                for (Integer integer : dim) {
                    InitVal offset = new InitVal(new ArrayList<>(), IntSysYType.construct());
                    offset.setName(String.valueOf(num % integer));
                    instrIndex.add(1, offset);
                    num /= integer;
                }
                buider.buildGep(array, instrIndex, bb);
                buider.buildStoreInstr(element, bb.getList().getLast().getValue(),bb);
            }
            i++;
            //value.addUse(visitVar(child, bb, function, locals));
        }
        return value;
    }

    public Value visitArray(Node node) {    //获得数组的初始化值
        User value;
        if (((ArraySysYType)((MyArray) node).getType()).getParam() instanceof IntSysYType) {
            value = new InitVal.SysYIntArray(new ArrayList<>(), ((MyArray) node).getType());
        } else {
            value = new InitVal.SysYFloatArray(new ArrayList<>(), ((MyArray) node).getType());
        }
        for (Node child : node.getChildren()) {
            value.addUse(visitVar(child));
        }
        return value;
    }

    private BinaryInstr.Type typeTrans(Operation.Operator operator, SysYType type, boolean notFlag) {   //tir和llvm的指令类型转换
        switch (operator) {
            case Add -> {
               if (!(type.equals(FloatSysYType.construct()))) {
                    return BinaryInstr.Type.Add;
               } else {
                   return BinaryInstr.Type.Fadd;
               }

            }
            case Sub -> {
                if (!(type.equals(FloatSysYType.construct()))) {
                    return BinaryInstr.Type.Sub;
                } else {
                    return BinaryInstr.Type.Fsub;
                }
            }
            case Mul -> {
                if (!(type.equals(FloatSysYType.construct()))) {
                    return BinaryInstr.Type.Mul;
                } else {
                    return BinaryInstr.Type.Fmul;
                }
            }
            case Div -> {
                if (!(type.equals(FloatSysYType.construct()))) {
                    return BinaryInstr.Type.Sdiv;
                } else {
                    return BinaryInstr.Type.Fdiv;
                }
            }
            case Mod -> {
                return BinaryInstr.Type.Srem;
            }
            case Not -> {
                return BinaryInstr.Type.Ne;
            }
            case Neg -> {
                if (!(type.equals(FloatSysYType.construct()))) {
                    return BinaryInstr.Type.Sub;
                } else {
                    return BinaryInstr.Type.FNeg;
                }
            }
            case Or -> {
                return BinaryInstr.Type.Or;
            }
            case And -> {
                return BinaryInstr.Type.And;
            }

            case Eq -> {
                if (notFlag) {
                    return BinaryInstr.Type.Ne;
                }
                return BinaryInstr.Type.Eq;
            }
            case Ne -> {
                if (notFlag) {
                    return BinaryInstr.Type.Eq;
                }
               return BinaryInstr.Type.Ne;
            }
            case Gt -> {
                if (notFlag) {
                    return BinaryInstr.Type.Le;
                }
                return BinaryInstr.Type.Gt;
            }
            case Ge -> {
                if (notFlag) {
                    return BinaryInstr.Type.Lt;
                }
                return BinaryInstr.Type.Ge;
            }
            case Lt -> {
                if (notFlag) {
                    return BinaryInstr.Type.Ge;
                }
                return BinaryInstr.Type.Lt;
            }
            case Le -> {
                if (notFlag) {
                    return BinaryInstr.Type.Gt;
                }
                return BinaryInstr.Type.Le;
            }
            default -> {
                return null;
            }
        }
    }

    public SysYType transType(Value value) {
        if (value.getType() instanceof ConstSysYType) {
            return ((ConstSysYType)value.getType()).getParam();
        } else {
            return value.getType();
        }
    }

    public Value visitOperation(Node node, BasicBlock bb, Function function, HashMap<String, SysYType> locals) {
        Operation operation = (Operation) node;
        Operation.Operator operator = operation.getOperators().get(0);
        ArrayList<Node> children = operation.getChildren();
        switch (operator) {
            case Not,CastInt,CastFloat -> {
                Value value = visitVar(children.get(0), bb, function, locals);
                buider.buildBinaryInstr(typeTrans(operator, value.getType(), false), value, bb);
                return bb.getList().getLast().getValue();
            }
            case Neg -> {
                Value value = visitVar(children.get(0), bb, function, locals);
                BinaryInstr.Type type = typeTrans(operator, value.getType(), false);
                if (type.equals(BinaryInstr.Type.Sub)) {
                    buider.buildBinaryInstr(type, visitVar(new NumberInt(0, null),bb, function, locals), value, bb);
                } else {
                    buider.buildBinaryInstr(type, value, bb);
                }
                return bb.getList().getLast().getValue();
            }
            default -> {
                Value ls = visitVar(children.get(0), bb, function, locals);
                Value rh = visitVar(children.get(1), bb, function, locals);
                if (!transType(ls).equals(transType(rh))) {
                    if (transType(ls).equals(IntSysYType.construct())) {
                        buider.buildCastInstr(ls, transType(rh), bb);
                        rh = bb.getList().getLast().getValue();
                    } else {
                        buider.buildCastInstr(rh, transType(ls), bb);
                        rh = bb.getList().getLast().getValue();
                    }
                }
                buider.buildBinaryInstr(typeTrans(operator, transType(ls), false), ls, rh, bb);
                return bb.getList().getLast().getValue();
            }
        }
    }

    private Value visitVar(Node node, BasicBlock bb, Function function, HashMap<String, SysYType> locals,
                           Value array) {
        //只用于访问数组
        Value value = null;
        if (node instanceof Ident) {
            String name = ((Ident) node).getIdent();
            value = function.findLocalIr(name);
            if (value != null) {
                if (value.getType() instanceof PointerSysYType) {
                    buider.buildLoadInstr(value, bb);
                    return bb.getList().getLast().getValue();
                }
                return value;
            }
            value = compileUnit.findGlobal("@" + name);
            if (value != null) {
                buider.buildLoadInstr(value, bb);
                return bb.getList().getLast().getValue();
            }
            value = new InitVal(null, globals.get(name));
        } else if (node instanceof NumberInt) {
            value = new InitVal(new ArrayList<>(), ConstSysYType.construct(IntSysYType.construct()));
            value.setName(String.valueOf(((NumberInt) node).getValue()));
        } else if (node instanceof MyArray) {
            value = visitArray(node, bb, function, locals, array);
        } else if (node instanceof NumberFloat) {
            value = new InitVal(new ArrayList<>(), ConstSysYType.construct(FloatSysYType.construct()));
            value.setName(String.valueOf(((NumberFloat) node).getValue()));
        } else if (node instanceof Operation) {
            return visitOperation(node, bb, function, locals);
        } else if (node instanceof FunCall) {
            value = visitFunCall(node, bb, function, locals);
        } else if (node instanceof Index) {
            ArrayList<Value> indexs = new ArrayList<>();
            indexs.add(buider.getZero());
            value = visitIndex(node, bb, function, locals, indexs);
        }
        return value;

    }

    private Value visitParam(Node node, BasicBlock bb, Function function, HashMap<String, SysYType> locals) {
        Value value = null;
        if (node instanceof Ident) {
            String name = ((Ident) node).getIdent();
            value = function.findLocalIr(name);
            if (value != null) {
                if (value.getType() instanceof PointerSysYType) {
                    PointerSysYType point = (PointerSysYType) value.getType();
                    if (!(point.getParam() instanceof ArraySysYType)) {
                        buider.buildLoadInstr(value, bb);
                        return bb.getList().getLast().getValue();
                    } else {
                        ArrayList<Value> indexs = new ArrayList<>();
                        indexs.add(buider.getZero());
                        indexs.add(buider.getZero());
                        buider.buildGep(value, indexs, bb);
                        return bb.getList().getLast().getValue();
                    }
                }
                return value;
            }
            value = compileUnit.findGlobal("@" + name);
            if (value != null) {
                if (value.getType() instanceof PointerSysYType) {
                    PointerSysYType point = (PointerSysYType) value.getType();
                    if (!(point.getParam() instanceof ArraySysYType)) {
                        buider.buildLoadInstr(value, bb);
                        return bb.getList().getLast().getValue();
                    } else {
                        ArrayList<Value> indexs = new ArrayList<>();
                        indexs.add(buider.getZero());
                        indexs.add(buider.getZero());
                        buider.buildGep(value, indexs, bb);
                        return bb.getList().getLast().getValue();
                    }
                }
                buider.buildLoadInstr(value, bb);
                return bb.getList().getLast().getValue();
            }
            value = new InitVal(null, globals.get(name));
        } else if (node instanceof NumberInt) {
            value = new InitVal(new ArrayList<>(), ConstSysYType.construct(IntSysYType.construct()));
            value.setName(String.valueOf(((NumberInt) node).getValue()));
        } else if (node instanceof MyArray) {
            ArrayList<Integer> list = new ArrayList<>();
            list.add(0);
            value = visitArray(node);
        } else if (node instanceof NumberFloat) {
            value = new InitVal(new ArrayList<>(), ConstSysYType.construct(FloatSysYType.construct()));
            value.setName(String.valueOf(((NumberFloat) node).getValue()));
        } else if (node instanceof Operation) {
            return visitOperation(node, bb, function, locals);
        } else if (node instanceof FunCall) {
            value = visitFunCall(node, bb, function, locals);
        } else if (node instanceof Index) {
            ArrayList<Value> indexs = new ArrayList<>();
            indexs.add(buider.getZero());
            indexs.add(buider.getZero());
            value = visitParamIndex(node, bb, function, locals, indexs);
        }
        return value;

    }


    private Value visitVar(Node node, BasicBlock bb, Function function, HashMap<String, SysYType> locals) {
        Value value = null;
        if (node instanceof Ident) {
            String name = ((Ident) node).getIdent();
            value = function.findLocalIr(name);
            if (value != null) {
                if (value.getType() instanceof PointerSysYType) {
                    buider.buildLoadInstr(value, bb);
                    return bb.getList().getLast().getValue();
                }
                return value;
            }
            value = compileUnit.findGlobal("@" + name);
            if (value != null) {
                buider.buildLoadInstr(value, bb);
                return bb.getList().getLast().getValue();
            }
            value = new InitVal(null, globals.get(name));
        } else if (node instanceof NumberInt) {
            value = new InitVal(new ArrayList<>(), ConstSysYType.construct(IntSysYType.construct()));
            value.setName(String.valueOf(((NumberInt) node).getValue()));
        } else if (node instanceof MyArray) {
            ArrayList<Integer> list = new ArrayList<>();
            list.add(0);
            value = visitArray(node);
        } else if (node instanceof NumberFloat) {
            value = new InitVal(new ArrayList<>(), ConstSysYType.construct(FloatSysYType.construct()));
            value.setName(String.valueOf(((NumberFloat) node).getValue()));
        } else if (node instanceof Operation) {
            return visitOperation(node, bb, function, locals);
        } else if (node instanceof FunCall) {
            value = visitFunCall(node, bb, function, locals);
        } else if (node instanceof Index) {
            ArrayList<Value> indexs = new ArrayList<>();
            indexs.add(buider.getZero());
            value = visitIndex(node, bb, function, locals, indexs);
        }
        return value;

    }

    private Value visitIndexEle(Node node, BasicBlock bb, Function function, HashMap<String,
            SysYType> locals, ArrayList<Value> lastIndexs) {
        ArrayList<Node> children = node.getChildren();
        Value offset = visitVar(children.get(1), bb, function, locals);
        //NumberInt offset = (NumberInt) children.get(1);
        // lastIndexs.add(offset.getValue());
        lastIndexs.add(1, offset);
        if (children.get(0) instanceof Index) {
            return visitIndexEle(children.get(0), bb, function, locals, lastIndexs);
        } else {
            Ident ident = (Ident) children.get(0);
            Value value = function.findLocalIr(ident.getIdent());
            if (value == null) {
                value = compileUnit.findGlobal("@" + ident.getIdent());
            }
            buider.buildGep(value, lastIndexs, bb);
            //buider.buildLoadInstr(bb.getList().getLast().getValue(), bb);
            return bb.getList().getLast().getValue();
        }

    }

    private Value visitIndex(Node node, BasicBlock bb, Function function, HashMap<String,
            SysYType> locals, ArrayList<Value> lastIndexs){
        ArrayList<Node> children = node.getChildren();
        Value offset = visitVar(children.get(1), bb, function, locals);
        //NumberInt offset = (NumberInt) children.get(1);
       // lastIndexs.add(offset.getValue());
        lastIndexs.add(1, offset);
        if (children.get(0) instanceof Index) {
             return visitIndex(children.get(0), bb, function, locals, lastIndexs);
        } else {
            Ident ident = (Ident) children.get(0);
            Value value = function.findLocalIr(ident.getIdent());
            if (value == null) {
                value = compileUnit.findGlobal("@" + ident.getIdent());
            }
            buider.buildGep(value, lastIndexs, bb);
            buider.buildLoadInstr(bb.getList().getLast().getValue(), bb);
            return bb.getList().getLast().getValue();
        }
    }
    private Value visitParamIndex(Node node, BasicBlock bb, Function function, HashMap<String,
            SysYType> locals, ArrayList<Value> lastIndexs){
        ArrayList<Node> children = node.getChildren();
        Value offset = visitVar(children.get(1), bb, function, locals);
        //NumberInt offset = (NumberInt) children.get(1);
        // lastIndexs.add(offset.getValue());
        lastIndexs.add(1, offset);
        if (children.get(0) instanceof Index) {
            return visitParamIndex(children.get(0), bb, function, locals, lastIndexs);
        } else {
            Ident ident = (Ident) children.get(0);
            Value value = function.findLocalIr(ident.getIdent());
            if (value == null) {
                value = compileUnit.findGlobal("@" + ident.getIdent());
            }
            buider.buildGep(value, lastIndexs, bb);
     //       buider.buildLoadInstr();
            return bb.getList().getLast().getValue();
        }
    }

    private Value visitVar(Node node) {
        Value value = null;
        if (node instanceof Ident) {
            String name = ((Ident) node).getIdent();
            value = buider.getInitVal(globals.get(name));
                    new InitVal(null, globals.get(name));
        } else if (node instanceof NumberInt) {
            value = new InitVal(null, ConstSysYType.construct(IntSysYType.construct()));
            value.setName(String.valueOf(((NumberInt) node).getValue()));
        } else if (node instanceof MyArray) {
            value = visitArray(node);
        } else if (node instanceof NumberFloat) {
            value = new InitVal(null, ConstSysYType.construct(FloatSysYType.construct()));
            value.setName(String.valueOf(((NumberFloat) node).getValue()));
        } else if (node instanceof Index) {
            System.out.println();
        }
//        module.addVar(value);
        return value;
    }

    private Value visitPtr(Node node, BasicBlock bb, Function function, HashMap<String, SysYType> locals) {
        String name = ((Ident) node).getIdent();
        Value oldValue = function.findLocalIr(name);
        if (oldValue != null) {
            return oldValue;
        }

        SysYType type;
        if (locals.containsKey(name)) {
            type = locals.get(name);
        } else {
            return compileUnit.findGlobal("@" + name);
        }
        Instr value = buider.getAllocaInstr(PointerSysYType.construct(type));
        bb.addLast(value);
        function.addLocal(name, value);
        //      module.addVar(value);
        return value;
    }

    private GlobalValue visitGlobal(Node node) {
        return buider.getGlobalValue(globals.get(((Ident) node).getIdent()), "@" + ((Ident) node).getIdent());
    }

    private GlobalValue visitLocalArray(Node node, HashMap<String, SysYType> locals) {
        SysYType type = locals.get(((Ident) node).getIdent());
        if (type instanceof ConstSysYType) {
            type = ((ConstSysYType) type).getParam();
        }
        return buider.getGlobalValue(type, NameSpace.getLocalArray());
    }


    private Value visitFunCall(Node node, BasicBlock bb, Function function, HashMap<String, SysYType> locals) {
        ArrayList<Value> params = new ArrayList<>();
        FunCall funCall = (FunCall) node;
        Function callFunc = compileUnit.findFunc("@" + funCall.getName());
        if (funCall.getName().equals("getstr")) {
            System.out.println();
        }
        for (Node child : node.getChildren()) {
            Value value = visitParam(child, bb, function, locals);
            params.add(value);
        }
        buider.buildCallInstr(callFunc, params, bb);
        return bb.getList().getLast().getValue();
    }

    private void visitOrOp(Node node, BasicBlock bb, Function function, HashMap<String, SysYType> locals,
                           BasicBlock ifBlock,BasicBlock endBb, boolean notFlag) {
        //处理只有变量的情况
        if (node instanceof Ident || node instanceof NumberInt || node instanceof FunCall || node instanceof Index) {
            Value ls = visitVar(node, bb, function, locals);
            Value rh = buider.getZero();
            BinaryInstr.Type type = notFlag ? BinaryInstr.Type.Eq : BinaryInstr.Type.Ne;
            buider.buildCmpInstr(type, ls, rh, bb);
            buider.buildBrInstr(bb.getList().getLast().getValue(), ifBlock, endBb, bb);
            return;
        }
        Operation operation = (Operation) node;
        Operation.Operator operator = operation.getOperators().get(0);
        ArrayList<Node> children = node.getChildren();
        if (operator.equals(Operation.Operator.And)) {
            BasicBlock andBlock = buider.getBasicBlock();
            visitOrOp(children.get(0), bb, function, locals, andBlock, endBb, notFlag);
            visitOrOp(children.get(1), andBlock, function, locals, ifBlock, endBb, notFlag);
            function.addLast(andBlock);
        } else if (operator.equals(Operation.Operator.Or)) {
            BasicBlock orBlock = buider.getBasicBlock();
            visitOrOp(children.get(0), bb, function, locals, ifBlock, orBlock, notFlag);
            visitOrOp(children.get(1), orBlock, function, locals, ifBlock, endBb, notFlag);
            function.addLast(orBlock);
        } else if (operator.equals(Operation.Operator.Not)) {
            visitOrOp(children.get(0), bb, function, locals, ifBlock, endBb, !notFlag);
        } else if (operator.equals(Operation.Operator.Neg)){
            visitOrOp(children.get(0), bb, function, locals, ifBlock, endBb, notFlag);
        } else {
            Value ls = visitVar(node.getChildren().get(0), bb, function, locals);
            Value rh = visitVar(node.getChildren().get(1), bb, function, locals);
            buider.buildCmpInstr(typeTrans(operator, ls.getType(), notFlag), ls, rh, bb);
            buider.buildBrInstr(bb.getList().getLast().getValue(), ifBlock, endBb, bb);
        }
    }

    private BasicBlock visitCondition(Node node, BasicBlock bb, Function function, HashMap<String, SysYType> locals,
                                      BasicBlock lastEndBlock, BasicBlock loopCondition) {
        ArrayList<Node> children = node.getChildren();
        BasicBlock endBb = buider.getBasicBlock();
        if (children.size() == 2) {
            BasicBlock ifBlock = buider.getBasicBlock();
            visitOrOp(children.get(0), bb, function, locals, ifBlock, endBb, false);
            function.addLast(ifBlock);
            BasicBlock curenv = visit(children.get(1), function, locals, ifBlock, lastEndBlock, loopCondition);
            if (!curenv.isEnd()) {
                buider.buildBrInstr(endBb, curenv);
                curenv.setEnd(true);
            }
        } else if (children.size() == 3) {
            BasicBlock ifBlock = buider.getBasicBlock();
            BasicBlock elseBlock = buider.getBasicBlock();
            visitOrOp(children.get(0), bb, function, locals, ifBlock, elseBlock, false);
            function.addLast(ifBlock);
            BasicBlock curenv = visit(children.get(1), function, locals, ifBlock, lastEndBlock, loopCondition);
            if (!curenv.isEnd()) {
                buider.buildBrInstr(endBb, curenv);
                curenv.setEnd(true);
            }
            if (!(children.get(2) instanceof Condition)) {
                curenv = visit(children.get(2), function, locals, elseBlock, lastEndBlock, loopCondition);
            } else {
                curenv = visitCondition(children.get(2), elseBlock, function, locals, lastEndBlock, loopCondition);
                function.addLast(curenv);
            }
            function.addLast(elseBlock);
            if (!curenv.isEnd()) {
                buider.buildBrInstr(endBb, curenv);
                curenv.setEnd(true);
            }
        }
        return endBb;
    }

    private BasicBlock visitLoop(Node node, Function function, HashMap<String, SysYType> locals,
                                 BasicBlock conditionBb) {
        ArrayList<Node> children = node.getChildren();
        BasicBlock endBb = buider.getBasicBlock();
        BasicBlock loopBodyBb = buider.getBasicBlock();
        visitOrOp(children.get(0), conditionBb, function, locals, loopBodyBb, endBb, false);
        function.addLast(loopBodyBb);

        BasicBlock curenv = visit(children.get(1), function, locals, loopBodyBb, endBb, conditionBb);
        if (!curenv.isEnd()) {
            buider.buildBrInstr(conditionBb, curenv);
            curenv.setEnd(true);
        }
        return endBb;
    }

    private BasicBlock visit(Node node, Function function, HashMap<String, SysYType> locals, BasicBlock bb,
                             BasicBlock endBlock, BasicBlock loopCondtion) {
        BasicBlock curenBb = bb;
        ArrayList<Node> itList;
        if (node instanceof Sequence) {
            itList = node.getChildren();
        } else {
            itList = new ArrayList<>();
            itList.add(node);
        }
        for (Node child : itList) {
            if (child instanceof Assign) {
                Assign assign = (Assign) child;
                ArrayList<Node> list = child.getChildren();
                if (list.get(0) instanceof Ident) {
                    SysYType assignType = locals.get(((Ident) list.get(0)).getIdent());
                    if ((assignType instanceof ArraySysYType ||
                            (assignType instanceof ConstSysYType && ((ConstSysYType) assignType).getParam() instanceof ArraySysYType))) {
                        GlobalValue ls = visitLocalArray(list.get(0), locals);
                        Value rh = visitVar(list.get(1), curenBb, function, locals, ls);
                        ls.addUse(rh);
                        compileUnit.addVar(ls.getName(), ls);
                        function.addLocal(((Ident) list.get(0)).getIdent(), ls);
                        module.addVar(ls);
                        continue;
                    }
                }
                Value ls;
                if (list.get(0) instanceof Index) {
                    ArrayList<Value> indexs = new ArrayList<>();
                    indexs.add(buider.getZero());
                    ls = visitIndexEle(list.get(0), curenBb, function, locals, indexs);
                } else {
                    ls = visitPtr(list.get(0), curenBb, function, locals);
                }
                //Value ls = visitPtr(list.get(0), curenBb, function, locals);
                Value rh = visitVar(list.get(1), curenBb, function, locals);
                if (!rh.getType().equals(((PointerSysYType) ls.getType()).getParam()) && (
                        !(rh.getType() instanceof ConstSysYType) || (rh.getType() instanceof ConstSysYType &&
                                !((ConstSysYType)(rh.getType())).getParam().equals(((PointerSysYType) ls.getType()).getParam()))
                        )) {
                    buider.buildCastInstr(rh, ((PointerSysYType) ls.getType()).getParam(), curenBb);
                    rh = curenBb.getList().getLast().getValue();
                }
                buider.buildStoreInstr(rh, ls, curenBb);
            } else if (child instanceof Return) {
                ArrayList<Node> list = child.getChildren();
                curenBb.setEnd(true);
                if (list.size() != 0) {
                    Value ret = visitVar(list.get(0), curenBb, function, locals);
                    //返回值类型转化待解决
                    buider.buildStoreInstr(ret, function.getReturnValue(), curenBb);
                }
                buider.buildBrInstr(function.getReturnBlock(), curenBb);
           //     buider.buildRetInstr(ret, curenBb);
            } else if (child instanceof FunCall) {
                visitFunCall(child, curenBb, function, locals);
            } else if (child instanceof Loop) {
                BasicBlock conditionBb = buider.getBasicBlock();
                curenBb.addUser(function);
                function.addLast(conditionBb);
                buider.buildBrInstr(conditionBb, curenBb);
                curenBb = visitLoop(child, function, locals, conditionBb);
                function.addLast(curenBb);
            } else if (child instanceof Condition) {
                curenBb = visitCondition(child, curenBb, function, locals, endBlock, loopCondtion);
                function.addLast(curenBb);
            } else if (child instanceof Break) {
                buider.buildBrInstr(endBlock, curenBb);
                curenBb.setEnd(true);
            } else if (child instanceof Continue) {
                buider.buildBrInstr(loopCondtion, curenBb);
                curenBb.setEnd(true);
            } else if (child instanceof Sequence) {
                curenBb = visit(child, function, locals, curenBb, endBlock, loopCondtion);
            }

        }
        return curenBb;
    }

    public MyVisitor(Root root) {
        this.root = root;
        this.globals = root.getGlobals();
        this.functions = root.getFunctions();
    }

    public MyModule getModule() {
        return module;
    }

    public void setName() {
        module.getGlobals().forEach(value -> {
           // if (!(value.getType() instanceof ConstSysYType))
         //   value.setName(NameSpace.getName());
        });
    }

    public void visitFunction(Node node) {
        FuncDef funcDef = (FuncDef)node;
        HashMap<String, SysYType> locals = funcDef.getLocals();
        Function function = new Function(functions.get(funcDef.getName()));
        function.setName("@" + funcDef.getName());
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);
        BasicBlock bb = buider.getBasicBlock();
        function.addFirst(bb);
        ArrayList<String> params = funcDef.getParameter();
        //为函数参数分配空间（alloca,store）
        for (String param : params) {
            if (!(locals.get(param) instanceof ArraySysYType)) {
                Instr instr = buider.getAllocaInstr(PointerSysYType.construct(locals.get(param)));
                bb.addLast(instr);
                function.addLocal(param, instr);
            } else {
                ArraySysYType arraySysYType = (ArraySysYType) locals.get(param);
                Instr instr = buider.getAllocaInstr(PointerSysYType.construct(PointerSysYType.construct(arraySysYType.getInner())));
                bb.addLast(instr);
                function.addLocal(param, instr);
            }
        }
        //为临时变量分配空间
        for (String name : locals.keySet()) {
            SysYType type = locals.get(name);
            if (function.containsLocal(name)) {
                continue;
             //   type = function.findLocalIr(name).getType();
            }
            Instr value = buider.getAllocaInstr(PointerSysYType.construct(type));
            bb.addLast(value);
            function.addLocal(name, value);
        }
        //设置函数初值
        for (String param : params) {
            if (!(locals.get(param) instanceof ArraySysYType)) {
                InitVal initVal = new InitVal(null, locals.get(param));
                function.addParam(initVal);
                Value ptr = function.findLocalIr(param);
                buider.buildStoreInstr(initVal, ptr, bb);
                ptr.addUser(bb.getList().getLast().getValue());
            } else {
                ArraySysYType arraySysYType = (ArraySysYType) locals.get(param);
                InitVal initVal = new InitVal(null, PointerSysYType.construct(arraySysYType.getInner()));
                function.addParam(initVal);
                Value ptr = function.findLocalIr(param);
                buider.buildStoreInstr(initVal, ptr, bb);
            }
        }
        //为返回值分配空间
        PointerSysYType returnType = PointerSysYType.construct(((FunctionSysYType)function.getType()).getRet());
        BasicBlock returnBlock = buider.getBasicBlock();
        function.setReturnBlock(returnBlock);
        if (!returnType.getParam().equals(VoidSysYType.construct())) {
            Instr returnValue = buider.getAllocaInstr(PointerSysYType.construct(((FunctionSysYType) function.getType()).getRet()));
            bb.addLast(returnValue);
            function.setReturnValue(returnValue);
        }
        //typeof node.getChildren().get(0) == Seq
        BasicBlock endBlock = visit(node.getChildren().get(0), function, locals, bb, null, null);
        //设置返回值
        function.addReturn(buider, endBlock);
    }

    public void addLib() {
        Function function = new Function(IntSysYType.construct());
        function.setName("@getint");
        function.setRuntime(true);
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);

        function = new Function(FloatSysYType.construct());
        function.setName("@getfloat");
        function.setRuntime(true);
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);

        function = new Function(VoidSysYType.construct());
        function.setName("@before_main");
        function.setRuntime(true);
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);

        function = new Function(VoidSysYType.construct());
        function.setName("@after_main");
        function.setRuntime(true);
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);

        function = new Function(FloatSysYType.construct());
        function.setName("@getfarray");
        function.setRuntime(true);
        InitVal prama = buider.getInitVal(PointerSysYType.construct(FloatSysYType.construct()));
        prama.setName("");
        function.addParam(prama);
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);

        function = new Function(IntSysYType.construct());
        function.setName("@getch");
        function.setRuntime(true);
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);

        function = new Function(IntSysYType.construct());
        function.setName("@putint");
        function.setRuntime(true);
        prama = buider.getInitVal(IntSysYType.construct());
        prama.setName("");
        function.addParam(prama);
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);

        function = new Function(IntSysYType.construct());
        function.setName("@putch");
        function.setRuntime(true);
        prama = buider.getInitVal(IntSysYType.construct());
        prama.setName("");
        function.addParam(prama);
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);

        function = new Function(IntSysYType.construct());
        function.setName("@putarray");
        function.setRuntime(true);
        prama = buider.getInitVal(IntSysYType.construct());
        prama.setName("");
        function.addParam(prama);
        prama = buider.getInitVal(PointerSysYType.construct(IntSysYType.construct()));
        prama.setName("");
        function.addParam(prama);
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);

        function = new Function(IntSysYType.construct());
        function.setName("@putf");
        function.setRuntime(true);
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);

        function = new Function(IntSysYType.construct());
        function.setName("@_sysy_starttime");
        function.setRuntime(true);
        prama = buider.getInitVal(IntSysYType.construct());
        prama.setName("");
        function.addParam(prama);
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);

        function = new Function(IntSysYType.construct());
        function.setName("@_sysy_stoptime");
        function.setRuntime(true);
        prama = buider.getInitVal(IntSysYType.construct());
        prama.setName("");
        function.addParam(prama);
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);

        function = new Function(IntSysYType.construct());
        function.setName("@getarray");
        function.setRuntime(true);
        prama = buider.getInitVal(PointerSysYType.construct(IntSysYType.construct()));
        prama.setName("");
        function.addParam(prama);
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);

        function = new Function(IntSysYType.construct());
        function.setName("@putfloat");
        function.setRuntime(true);
        prama = buider.getInitVal(FloatSysYType.construct());
        prama.setName("");
        function.addParam(prama);
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);

        function = new Function(IntSysYType.construct());
        function.setName("@putfarray");
        function.setRuntime(true);
        prama = buider.getInitVal(PointerSysYType.construct(FloatSysYType.construct()));
        prama.setName("");
        function.addParam(prama);
        module.addFunc(function);
        compileUnit.addFunction(function.getName(), function);

    }

    public void compiler() {
        addLib();
        for (String s : globals.keySet()) {
            if (compileUnit.findGlobal("@" + s) == null) {
                GlobalValue value = buider.getGlobalValue(globals.get(s), "@" + s);
                InitVal initVal = new InitVal(new ArrayList<>(), globals.get(s));
                if (!(globals.get(s) instanceof ArraySysYType)) {
                    initVal.setName("0");
                } else {
                   initVal.setName("zeroinitializer");
                }
                value.addUse(initVal);
                compileUnit.addVar(value.getName(), value);
                module.addVar(value);
            }
        }
        for (Node child : root.getChildren()) {
            if (child instanceof Assign) {
                Assign assign = (Assign) child;
                ArrayList<Node> list = child.getChildren();
                GlobalValue ls = visitGlobal(list.get(0));
                Value rh = visitVar(list.get(1));
                ls.addUse(rh);
                compileUnit.addVar(ls.getName(), ls);
                module.addVar(ls);
                //  buider.buildStoreInstr(rh, ls, bb);
            } else if (child instanceof FuncDef) {
                visitFunction(child);
            }

        }
        //int x = 0;
        module.setName();

    }

    public void setName(BasicBlock bb) {
       // bb.setName(NameSpace.getName());
        bb.getList().forEach(z->{

                }
        );
    }

    private Function getFunction(Node node) {
        return null;
    }

    private BasicBlock getBasicBlock(Node node) {
        return null;
    }

}
