package ir.instruction;

import ir.type.SysYType;
import ir.value.User;
import ir.value.Value;

import java.util.ArrayList;

public class BinaryInstr extends Instr {

    private final Type type;
    private Value left;
    private Value right;
    private Type cmp;

    public BinaryInstr(ArrayList<User> users, ArrayList<Value> uses, Type type) {
        super(users, uses);
        this.type = type;
    }

    public BinaryInstr(Type tag, SysYType type,Value l) {
        super(type,"");
        ArrayList<Value> uses = new ArrayList<>();
        ArrayList<User> users = new ArrayList<>();
        this.left = l;
        uses.add(l);
        this.type = tag;
    }


    public BinaryInstr(Type tag, SysYType type,Value l, Value r) {
        super(type,"");
        ArrayList<Value> uses = new ArrayList<>();
        ArrayList<User> users = new ArrayList<>();
        this.left = l;
        this.right = r;
        uses.add(l);
        uses.add(r);
        this.type = tag;
    }
    public BinaryInstr(Type tag, SysYType type, Value l, Value r, Type cmp) {
        super(type,"");
        ArrayList<Value> uses = new ArrayList<>();
        ArrayList<User> users = new ArrayList<>();
        this.left = l;
        this.right = r;
        uses.add(l);
        uses.add(r);
        this.type = tag;
        this.cmp = cmp;
    }

    public enum Type {
        Add,
        Sub,
        Mul,
        Sdiv,
        Fadd,
        Fsub,
        Fmul,
        Fdiv,
        Srem, // 符合C关于模除的定义 ISO C99 p82
        And,
        Or, // 对于SysY来说只有短路运算的逻辑运算，但是貌似只有表达式中有函数调用才会产生副作用，可以优化？
        ICmp,
        FCmp,
        Xor,
        FNeg, //其实这不是Binary Instruction
        Shl,
        Eq,
        Ne,
        Gt,
        Ge,
        Lt,
        Le,
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(getName()).append(" = ");
        sb.append(type.toString().toLowerCase()).append(" ");
        if (type == Type.ICmp || type == Type.FCmp) {
            if (cmp != Type.Eq && cmp != Type.Ne ) {
                sb.append("s").append(cmp.toString().toLowerCase());
                sb.append(" ");
            } else {
                sb.append(cmp.toString().toLowerCase());
                sb.append(" ");
            }
        }
        sb.append(getType()).append(" ");
        sb.append(left.getName()).append(", ").append(right.getName());
        return sb.toString();
    }

    enum CmpCond {

    }
}
