package ir.instruction;

import ir.type.FunctionSysYType;
import ir.type.VoidSysYType;
import ir.value.Function;
import ir.value.User;
import ir.value.Value;

import java.util.ArrayList;

public class CallInstr extends Instr {
    private Function function;
    private ArrayList<Value> params;
    private boolean isVoid;                 //判断类型是否为void，是否需要设置name

    public CallInstr(ArrayList<User> users, ArrayList<Value> uses) {
        super(users, uses);
    }

    public CallInstr(Function function, ArrayList<Value> params) {
        super(function.getType(), "");
        if (getType() instanceof FunctionSysYType) {
            setType(((FunctionSysYType) getType()).getRet());
        }
        isVoid = getType().equals(new VoidSysYType());
        this.function = function;
        this.params = params;
    }

    @Override
    public boolean CanNamed() {
        if (isVoid) {
            return false;
        }
        return super.CanNamed();
    }

    public void setVoid(boolean aVoid) {
        isVoid = aVoid;
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        if (!isVoid) {
            sb.append(getName()).append(" = ");
        }
        sb.append("call ");
        sb.append(getType());
        sb.append(" ").append(function.getName()).append("(");
        for (int i = 0; i < params.size(); i++) {
            sb.append(params.get(i).getType()).append(" ");
            sb.append(params.get(i).getName());
            if (i != params.size() -1) {
                sb.append(", ");
            }
        }
        sb.append(")");
        return sb.toString();
    }
}
