package ir.type;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 数组类型
 */
public class ArraySysYType implements SysYType {
    private final SysYType param;
    private final int size;

    public SysYType getParam() {
        return param;
    }

    /**
     * @param param IntType or FloatType
     * @param size 数组大小
     */
    public ArraySysYType(SysYType param, int size) {
        this.param = param;
        this.size = size;
    }

    /**
     *
     * @return 数组成员类型
     */
    public SysYType getInner() {
        return param;
    }
    public static ArraySysYType construct(SysYType inner, int size) {
        return new ArraySysYType(inner, size);
    }
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        ArraySysYType that = (ArraySysYType) o;
        return size == that.size && param.equals(that.param);
    }

    /**
     * 数组表示为嵌套的类型，例如 [[int,2],3]
     * 但是有时需要C风格的表示，如 int[3][2],
     * 对于上面的类型，返回 [3,2]
     * @return 展平的数组维数
     */
    public List<Integer> getDims() {
        var ret = new ArrayList<Integer>();
        ret.add(size);
        var t = getInner();
        while (t instanceof ArraySysYType) {
            var arr = (ArraySysYType) t;
            ret.add(arr.size);
            t = arr.getInner();
        }
        return ret;
    }

    @Override
    public String toString() {
        return "[" + size + " x " + param + "]";
    }

    @Override
    public int hashCode() {
        return Objects.hash(param, size);
    }

    public int getSize() {
        return size;
    }
}
