package ir.value;

import ir.instruction.Instr;
import util.MyList;
import util.MyNode;

import java.util.ArrayList;
import java.util.List;

public class BasicBlock extends User {

    public List<BasicBlock> getPrecBB() {
        return PrecBB;
    }

    public List<BasicBlock> getSuccBB() {
        return SuccBB;
    }
    public void clearPrecSuccInfo() {
        PrecBB.clear();
        SuccBB.clear();
    }

    public List<BasicBlock> getDominator() {
        return dominator;
    }

    public List<BasicBlock> getDominating() {
        return dominating;
    }


    public void clearDominationInfo() {
        dominator.clear();
        dominating.clear();
        idominator = null;
        df.clear();
    }
    // 紧前基本块
    private final List<BasicBlock> PrecBB = new ArrayList<>();

    // 后继基本块
    private final  List<BasicBlock> SuccBB = new ArrayList<>();

    public BasicBlock getIdominator() {
        return idominator;
    }

    public void setIdominator(BasicBlock idominator) {
        this.idominator = idominator;
    }

    // 直接支配
    private BasicBlock idominator = null;

    public List<BasicBlock> getDf() {
        return df;
    }

    private final List<BasicBlock> df = new ArrayList<>();

    // 支配的基本块
    private final List<BasicBlock> dominator = new ArrayList<>();

    // 被哪些基本块支配
    private final List<BasicBlock> dominating = new ArrayList<>();

    //标记是否已经设置退出指令
    private boolean isEnd = false;

    public boolean isEnd() {
        return isEnd;
    }

    public void setEnd(boolean end) {
        isEnd = end;
    }

    private final ArrayList<Instr> instrs = new ArrayList<>();

    private MyNode<BasicBlock> node;

    public MyNode<BasicBlock> getNode() {
        return node;
    }

    public void setNode(MyNode<BasicBlock> node) {
        this.node = node;
    }

    private MyList<Instr> list = new MyList<>();

    public BasicBlock(ArrayList<User> users, ArrayList<Value> uses) {
        super(users, uses);
        node = new MyNode<>(null, this, null);
    }

//    public BasicBlock(ArrayList<User> users) {
//        super(users);
//    }

    public ArrayList<Instr> getInstrs() {
        return instrs;
    }

    public void addInstr(Instr instr) {
        instrs.add(instr);
    }

    public void addFirst(Instr instr) {
        list.addFirst(instr.getNode());
    }

    public MyList<Instr> getList() {
        return list;
    }

    public void addLast(Instr instr) {
        list.addLast(instr.getNode());
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(getName().substring(1)).append(":\n");
        for (MyNode<Instr> myNode : list) {
            sb.append("\t").append(myNode.getValue().toString()).append("\n");
        }
        return sb.toString();
    }
}
