package ir.value;

import java.util.HashMap;
import java.util.HashSet;

/**
 * 表示一个编译文件，是个单例
 */
public class CompileUnit {
    /*
        全局变量和局部变量的作用域可以重叠，重叠部分局部变量优先；同名局
        部变量的作用域不能重叠；
        SysY 语言中变量名可以与函数名相同。
     */
    private final HashMap<String, Function>  funcSymbols = new HashMap<>(); // 全局函数符号表
    private final HashMap<String, GlobalValue>  globalVarSymbols = new HashMap<>(); // 全局变量符号表
    private final HashSet<InitVal> constants = new HashSet<>(); // 指令引用的常量
    private static final CompileUnit instance = new CompileUnit();

    public Value findGlobal(String name) {
        return globalVarSymbols.get(name);
    }

    public Function findFunc(String name) {
        return funcSymbols.get(name);
    }

    public static CompileUnit getInstance() {
        return instance;
    }

    public void addVar(String name, GlobalValue val) {
        globalVarSymbols.put(name, val);
    }

    public void addFunction(String name,Function function) {
        funcSymbols.put(name, function);
    }
}
