package ir.value;

import ir.MyIrBuilder;
import ir.instruction.Instr;
import ir.type.ConstSysYType;
import ir.type.SysYType;
import ir.type.VoidSysYType;
import util.MyList;
import util.MyNode;

import java.util.ArrayList;
import java.util.HashMap;

public class Function extends User{
    private ArrayList<BasicBlock> basicBlocks;
    private final SysYType type;
    private ArrayList<Value> params = new ArrayList<>();
    private MyList<BasicBlock> list = new MyList<>();
    private MyNode<Function> node;
    private HashMap<String, Value> irLoclas = new HashMap<>();
    private HashMap<String, Value> name2Value = new HashMap<>();
    private boolean isRuntime = false;
    private Instr returnValue;
    private BasicBlock returnBlock;


    public Instr getReturnValue() {
        return returnValue;
    }

    public BasicBlock getReturnBlock() {
        return returnBlock;
    }

    public void setReturnBlock(BasicBlock returnBlock) {
        this.returnBlock = returnBlock;
    }

    public void addReturn(MyIrBuilder builder, BasicBlock endBlock) {
        if (returnValue != null) {
            if (!endBlock.isEnd()) {
                builder.buildBrInstr(returnBlock, endBlock);
            }
            addLast(returnBlock);
            builder.buildLoadInstr(returnValue, returnBlock);
            builder.buildRetInstr(returnBlock.getList().getLast().getValue(), returnBlock);
        } else {
            if (!endBlock.isEnd()) {
                builder.buildBrInstr(returnBlock, endBlock);
            }
            addLast(returnBlock);
            builder.buildRetInstr(builder.getInitVal(VoidSysYType.construct()), returnBlock);
        }
    }

    public void setReturnValue(Instr returnValue) {
        this.returnValue = returnValue;
    }

    public void setRuntime(boolean runtime) {
        isRuntime = runtime;
    }

    public ArrayList<Value> getParams() {
        return params;
    }

    public void addParam(Value value) {
        params.add(value);
    }

    public Value findLocalIr(String name) {
        return irLoclas.get(name);
    }

    public void addLocal(String name, Value value) {
        irLoclas.put(name, value);
    }

    public boolean containsLocal(String name) {
        return irLoclas.containsKey(name);
    }

    public MyNode<Function> getNode() {
        return node;
    }

    public SysYType getType() {
        return type;
    }

    public void addFirst(BasicBlock basicBlock) {
        list.addFirst(basicBlock.getNode());
    }

    public MyList<BasicBlock> getList() {
        return list;
    }

    public void addLast(BasicBlock basicBlock) {
        list.addLast(basicBlock.getNode());
    }
    
    public Function(ArrayList<BasicBlock> basicBlocks, SysYType type) {
        super(type, "");
        this.basicBlocks = basicBlocks;
        this.type = type;
        node = new MyNode<>(null, this, null);
    }

    public Function(SysYType type) {
        super(type, "");
        this.basicBlocks = new ArrayList<>();
        this.type = type;
        node = new MyNode<>(null, this, null);
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        if (!isRuntime) {
            sb.append("define dso_local ");
        } else {
            sb.append("declare ");
        }
        sb.append(getType()).append(" ");
        sb.append(getName()).append(" ");
        sb.append("(");
        for (int i = 0; i < params.size(); i++) {
            sb.append(params.get(i).getType()).append(" ");
            if (!isRuntime) {
                sb.append(params.get(i).getName());
            }
            if (i != params.size() -1) {
                sb.append(", ");
            }
        }
        sb.append(")");
        if (!isRuntime) {
            sb.append("{\n");
            for (MyNode<BasicBlock> myNode : list) {
                sb.append(myNode.getValue().toString());
            }
            sb.append("}\n");
        }
        return sb.toString();
    }
}
