package ir.value;

import ir.type.ArraySysYType;
import ir.type.SysYType;
import treeir.TreeValue;

import java.util.ArrayList;

public class InitVal extends User{
    public InitVal(ArrayList<User> users, SysYType type) {
        super(users, type);
    }

    public InitVal(SysYType type, String name) {
        super(type, name);
    }

    public static class SysYFloat extends InitVal{
        private float val;

        public SysYFloat(ArrayList<User> users, SysYType type, float val) {
            super(users, type);
            this.val = val;
        }
    }
    public static class SysYInt extends InitVal{
        private int val;

        public SysYInt(ArrayList<User> users, SysYType type, int val) {
            super(users, type);
            this.val = val;
        }
    }
    public static class SysYFloatArray extends InitVal{
        private Object val;
        private ArrayList<Integer> dim; //维度

        public SysYFloatArray(ArrayList<User> users, SysYType type, Object val, ArrayList<Integer> dim) {
            super(users, type);
            this.val = val;
            this.dim = dim;
        }

        public SysYFloatArray(ArrayList<User> users, SysYType type) {
            super(users, type);
        }
    }
    public static class SysYIntArray extends InitVal{
        private Object val;
        private ArrayList<Integer> dim;

        public SysYIntArray(ArrayList<User> users, SysYType type, Object val, ArrayList<Integer> dim) {
            super(users, type);
            this.val = val;
            this.dim = dim;
        }

        public SysYIntArray(ArrayList<User> users, SysYType type) {
            super(users, type);
        }
    }
    public static class SysYString extends InitVal{
        private String string;

        public SysYString(ArrayList<User> users, SysYType type, String string) {
            super(users, type);
            this.string = string;
        }
    }

    private String usesToString(ArraySysYType type, int first) {
        StringBuilder sb = new StringBuilder();
        sb.append(type).append(" ");
        sb.append("[");
        if (!(type.getParam() instanceof ArraySysYType)) {
            for (int i = 0; i < type.getSize(); i++) {
                sb.append(uses.get(i + first));
                if (i != type.getSize() - 1) {
                    sb.append(", ");
                }
            }
        } else {
            for (int i = 0; i < type.getSize(); i++) {
                sb.append(usesToString((ArraySysYType) type.getParam(),
                        first + ((ArraySysYType) type.getParam()).getSize() * i));
                if (i != type.getSize() - 1) {
                    sb.append(", ");
                }
            }
        }
        sb.append("]");
        return sb.toString();
    }

    @Override
    public String toString() {
        if (!(this instanceof SysYIntArray || this instanceof SysYFloatArray)) {
            return getType() + " "  + getName();
        } else {
            return usesToString((ArraySysYType) getType(), 0);
        }
    }
}
