package treeir;

import java.util.ArrayList;
import java.util.Map;

import static treeir.Operation.Operator.*;

/**
 * Operation (Value,Value,...) -> Value
 * 算术运算是Operation, 函数也是Operation
 */
public class Operation extends Node{
    private final ArrayList<Operator> operators;
    private final OperatorType type;

    public Operation(ArrayList<Node> children, ArrayList<Operator> operators, Node parent, OperatorType type) {
        super(children, parent);
        this.operators = operators;
        this.type = type;
    }

    public OperatorType getType() {
        return type;
    }
    public void appendOp(Operator op) {
        //children.add(para);
        operators.add(op);
    }
    @Override
    public TreeValue evaluate(Map<String, TreeValue> env) {
        switch (operators.get(0)) {
            case Neg,Not,CastInt,CastFloat -> {
                return children.get(0).evaluate(env).op(operators.get(0));
            }
            default -> {
                var t = children.get(0).evaluate(env);
                for (int i = 1; i < children.size(); i++) {
                    t = t.op(children.get(i).evaluate(env), operators.get(i - 1));
                }
                return t;
            }

        }
    }
    public ArrayList<Node> getChildren() {
        return children;
    }

    public ArrayList<Operator> getOperators() {
        return operators;
    }
    enum OperatorType{
        Float,
        Int
    }
    public enum Operator{
        Neg,
        Not,
        Mul,
        Div,
        Mod,
        Add,
        Sub,
        Lt,
        Gt,
        Le,
        Ge,
        Eq,
        Ne,
        And,
        Or,
        CastInt, // 隐式转换
        CastFloat,

    }

    @Override
    public String toString() {
        switch (operators.get(0)) {
            case Neg,Not,CastInt,CastFloat -> {
                return String.format("(%s %s)", operators, children.get(0));
            }
            default -> {
                var sb = new StringBuilder();
                sb.append('(');
                sb.append(children.get(0));
                for (int i = 1; i < children.size(); i++) {
                    sb.append(' ').append(operators.get(i-1)).append(' ').append(children.get(i));
                }
                sb.append(')');
                return sb.toString();
            }
        }

    }
}
