package treeir;

import java.util.ArrayList;

/**
 * enum{
 *     intVal(int),
 *     floatVal(float),
 * }
 */
public class TreeValue {
    public int intVal;
    public float floatVal;

    public ArrayList<TreeValue> array;

    public ArrayList<Integer> dims;
    public int discrimination;

    public TreeValue(int intVal) {
        this.intVal = intVal;
        discrimination = 0;
    }

    public TreeValue(float floatVal) {
        this.floatVal = floatVal;
        discrimination = 1;
    }

    public TreeValue(ArrayList<TreeValue> array, ArrayList<Integer> dims) {
        this.array = array;
        discrimination = 2;
        this.dims = dims;
    }

    public TreeValue(int intVal,float floatVal) {
        this.floatVal = floatVal;
        this.intVal = intVal;
        discrimination = 0;
    }
    public void castFloat() {
        if (discrimination == 0) {
            discrimination = 1;
            floatVal = (float) intVal;
        }
    }

    public void castInt() {
        if (discrimination == 1) {
            discrimination = 0;
            intVal = (int) floatVal;
        }
    }

    /**
     * 算法类似C的指针与数组的运算。
     * 例如 int[3][2][5] -> int[2][5]
     * @param idx 偏移
     * @return 数组
     */
    public TreeValue index(int idx) {
        if (dims.size() == 1) {
            return array.get(idx);
        }
        var t = idx;
        var len = 1;
        var newDims = new ArrayList<Integer>();
        for (int i = 1; i < dims.size(); i++) {
            t *= dims.get(i);
            len *= dims.get(i);
            newDims.add(dims.get(i));
        }
        return new TreeValue(new ArrayList<>(array.subList(t,t+len)), newDims);
    }
    public TreeValue op(Operation.Operator operator) {
        switch (operator) {
            case Neg,Not,CastFloat,CastInt -> {
                return op(new TreeValue(0), operator);
            }
            default -> {
                assert false;
            }
        }
        return null;
    }
    public TreeValue op(TreeValue o, Operation.Operator operator) {
        switch (operator) {
            case Neg -> {
                if (discrimination == 0) {
                    return new TreeValue(-intVal);
                } else {
                    return new TreeValue(-floatVal);
                }
            }
            case Not -> {
                if (discrimination == 1) {
                    castInt();
                }
                if (intVal > 0) {
                    return new TreeValue(0);
                } else {
                    return new TreeValue(1);
                }
            }
            case Mod -> {
                if (discrimination == 1) {
                    castInt();
                }
                if (o.discrimination == 1) {
                    o.castInt();
                }
                return new TreeValue(intVal % o.intVal);
            }
            case Mul,Div,Add,Sub -> {
                if (discrimination == 1 || o.discrimination == 1) {
                    //cast to float
                    castFloat();
                    o.castFloat();
                    switch (operator) {
                        case Mul -> {
                            return new TreeValue(floatVal * o.floatVal);
                        }
                        case Div -> {
                            return new TreeValue(floatVal / o.floatVal);
                        }
                        case Add -> {
                            return new TreeValue(floatVal + o.floatVal);
                        }
                        case Sub -> {
                            return new TreeValue(floatVal - o.floatVal);
                        }
                    }
                }
                switch (operator) {
                    case Mul -> {
                        return new TreeValue(intVal * o.intVal);
                    }
                    case Div -> {
                        return new TreeValue(intVal / o.intVal);
                    }
                    case Add -> {
                        return new TreeValue(intVal + o.intVal);
                    }
                    case Sub -> {
                        return new TreeValue(intVal - o.intVal);
                    }
                }

            }
            case Lt,Gt,Le,Ge,Eq,Ne -> {
                if (discrimination == 1 || o.discrimination == 1) {
                    //cast to float
                    castFloat();
                    o.castFloat();
                    switch (operator) {
                        case Lt -> {
                            return new TreeValue(floatVal < o.floatVal?1:0);
                        }
                        case Gt -> {
                            return new TreeValue(floatVal > o.floatVal?1:0);
                        }
                        case Le -> {
                            return new TreeValue(floatVal <= o.floatVal?1:0);
                        }
                        case Ge -> {
                            return new TreeValue(floatVal >= o.floatVal?1:0);
                        }
                        case Eq -> {
                            return new TreeValue(floatVal == o.floatVal?1:0);
                        }
                        case Ne -> {
                            return new TreeValue(floatVal != o.floatVal?1:0);
                        }
                    }
                }
                switch (operator) {
                    case Lt -> {
                        return new TreeValue(intVal < o.intVal?1:0);
                    }
                    case Gt -> {
                        return new TreeValue(intVal > o.intVal?1:0);
                    }
                    case Le -> {
                        return new TreeValue(intVal <= o.intVal?1:0);
                    }
                    case Ge -> {
                        return new TreeValue(intVal >= o.intVal?1:0);
                    }
                    case Eq -> {
                        return new TreeValue(intVal == o.intVal?1:0);
                    }
                    case Ne -> {
                        return new TreeValue(intVal != o.intVal?1:0);
                    }
                }
            }
            case And,Or -> {
                castInt();
                o.castInt();
                if (operator == Operation.Operator.And && intVal > 0 && o.intVal > 0) {
                    return new TreeValue(1);
                }
                if (operator == Operation.Operator.Or && (intVal > 0 || o.intVal > 0)) {
                    return new TreeValue(1);
                }
                return new TreeValue(0);
            }
            case CastInt -> {
                castInt();
                return new TreeValue(intVal);
            }
            case CastFloat -> {
                castFloat();
                return new TreeValue(floatVal);
            }
        }
        assert false;
        return null;
    }

}
